@@ -12,7 +12,7 @@ import { expect } from 'chai';
 import { Application } from '@lumino/application';
 import { ContextMenu, Widget } from '@lumino/widgets';
 import { CommandRegistry } from '@lumino/commands';
-import { Token } from '@lumino/coreutils';
+import { PluginRegistry, Token } from '@lumino/coreutils';
 
 describe('@lumino/application', () => {
   describe('Application', () => {
@@ -28,6 +28,35 @@ describe('@lumino/application', () => {
         expect(app.contextMenu).to.be.instanceOf(ContextMenu);
         expect(app.shell).to.equal(shell);
       });
+
+      it('should accept an external plugin registry', async () => {
+        const shell = new Widget();
+        const pluginRegistry = new PluginRegistry();
+        const id1 = 'plugin1';
+        pluginRegistry.registerPlugin({
+          id: id1,
+          activate: () => {
+            // no-op
+          }
+        });
+        const id2 = 'plugin2';
+        pluginRegistry.registerPlugin({
+          id: id2,
+          activate: () => {
+            // no-op
+          }
+        });
+
+        const app = new Application({
+          shell,
+          pluginRegistry
+        });
+
+        await pluginRegistry.activatePlugin(id2);
+
+        expect(app.hasPlugin(id1)).to.be.true;
+        expect(app.isPluginActivated(id2)).to.be.true;
+      });
     });
 
     describe('#getPluginDescription', () => {
