@@ -27,6 +27,7 @@ export class TextRenderer extends CellRenderer {
     this.backgroundColor = options.backgroundColor || '';
     this.verticalAlignment = options.verticalAlignment || 'center';
     this.horizontalAlignment = options.horizontalAlignment || 'left';
+    this.horizontalPadding = options.horizontalPadding || 8;
     this.format = options.format || TextRenderer.formatGeneric();
     this.elideDirection = options.elideDirection || 'none';
     this.wrapText = options.wrapText || false;
@@ -57,6 +58,11 @@ export class TextRenderer extends CellRenderer {
    */
   readonly horizontalAlignment: CellRenderer.ConfigOption<TextRenderer.HorizontalAlignment>;
 
+  /**
+   * The horizontal alignment for the cell text.
+   */
+  readonly horizontalPadding: number;
+
   /**
    * The format function for the cell value.
    */
@@ -105,6 +111,13 @@ export class TextRenderer extends CellRenderer {
     gc.fillRect(config.x, config.y, config.width, config.height);
   }
 
+  /**
+   * Get the full text to be rendered by the cell.
+   */
+  getText(config: CellRenderer.CellConfig): string {
+    return this.format(config);
+  }
+
   /**
    * Draw the text for the cell.
    *
@@ -130,8 +143,7 @@ export class TextRenderer extends CellRenderer {
     }
 
     // Format the cell value to text.
-    let format = this.format;
-    let text = format(config);
+    let text = this.getText(config);
 
     // Bail if there is no text to draw.
     if (!text) {
@@ -185,7 +197,7 @@ export class TextRenderer extends CellRenderer {
     // Compute the X position for the text.
     switch (hAlign) {
       case 'left':
-        textX = config.x + 8;
+        textX = config.x + this.horizontalPadding;
         boxWidth = config.width - 14;
         break;
       case 'center':
@@ -193,7 +205,7 @@ export class TextRenderer extends CellRenderer {
         boxWidth = config.width;
         break;
       case 'right':
-        textX = config.x + config.width - 8;
+        textX = config.x + config.width - this.horizontalPadding;
         boxWidth = config.width - 14;
         break;
       default:
@@ -302,31 +314,35 @@ export class TextRenderer extends CellRenderer {
     }
 
     // Elide text that is too long
-    let elide = '\u2026';
+    const elide = '\u2026';
 
-    // Compute elided text
-    if (elideDirection === 'right') {
-      while (textWidth > boxWidth && text.length > 1) {
-        if (text.length > 4 && textWidth >= 2 * boxWidth) {
-          // If text width is substantially bigger, take half the string
-          text = text.substring(0, text.length / 2 + 1) + elide;
+    // Loop until text width fits box or only one character remains
+    while (textWidth > boxWidth && text.length > 1) {
+      // Convert text string to array for dealing with astral symbols
+      const textArr = [...text];
+
+      if (elideDirection === 'right') {
+        // If text width is substantially bigger, take half the string
+        if (textArr.length > 4 && textWidth >= 2 * boxWidth) {
+          text =
+            textArr.slice(0, Math.floor(textArr.length / 2 + 1)).join('') +
+            elide;
         } else {
           // Otherwise incrementally remove the last character
-          text = text.substring(0, text.length - 2) + elide;
+          text = textArr.slice(0, textArr.length - 2).join('') + elide;
         }
-        textWidth = gc.measureText(text).width;
-      }
-    } else {
-      while (textWidth > boxWidth && text.length > 1) {
-        if (text.length > 4 && textWidth >= 2 * boxWidth) {
-          // If text width is substantially bigger, take half the string
-          text = elide + text.substring(text.length / 2);
+      } else {
+        // If text width is substantially bigger, take half the string
+        if (textArr.length > 4 && textWidth >= 2 * boxWidth) {
+          text = elide + textArr.slice(Math.floor(textArr.length / 2)).join('');
         } else {
           // Otherwise incrementally remove the last character
-          text = elide + text.substring(2);
+          text = elide + textArr.slice(2).join('');
         }
-        textWidth = gc.measureText(text).width;
       }
+
+      // Measure new text width
+      textWidth = gc.measureText(text).width;
     }
 
     // Draw the text for the cell.
@@ -392,6 +408,13 @@ export namespace TextRenderer {
      */
     horizontalAlignment?: CellRenderer.ConfigOption<HorizontalAlignment>;
 
+    /**
+     * The horizontal padding for the cell text in pixels.
+     *
+     * The default is `8`.
+     */
+    horizontalPadding?: number;
+
     /**
      * The format function for the renderer.
      *
