@@ -446,6 +446,12 @@ export class Menu extends Widget {
    * fully fit on the screen. If it will not fit, it will be adjusted
    * to fit naturally on the screen.
    *
+   * The menu will be attached under the `host` element in the DOM
+   * (or `document.body` if `host` is `null`) and before the `ref`
+   * element (or as the last child of `host` if `ref` is `null`).
+   * The menu may be displayed outside of the `host` element
+   * following the rules of CSS absolute positioning.
+   *
    * This is a no-op if the menu is already attached to the DOM.
    */
   open(x: number, y: number, options: Menu.IOpenOptions = {}): void {
@@ -454,12 +460,26 @@ export class Menu extends Widget {
       return;
     }
 
-    // Extract the position options.
+    // Extract the menu options.
     let forceX = options.forceX || false;
     let forceY = options.forceY || false;
+    const host = options.host ?? null;
+    const ref = options.ref ?? null;
+    const horizontalAlignment =
+      options.horizontalAlignment ??
+      (document.documentElement.dir === 'rtl' ? 'right' : 'left');
 
     // Open the menu as a root menu.
-    Private.openRootMenu(this, x, y, forceX, forceY);
+    Private.openRootMenu(
+      this,
+      x,
+      y,
+      forceX,
+      forceY,
+      horizontalAlignment,
+      host,
+      ref
+    );
 
     // Activate the menu to accept keyboard input.
     this.activate();
@@ -986,6 +1006,28 @@ export namespace Menu {
      * The default is `false`.
      */
     forceY?: boolean;
+
+    /**
+     * The DOM node to use as the menu's host.
+     *
+     * If not specified then uses `document.body`.
+     */
+    host?: HTMLElement;
+
+    /**
+     * The child of `host` to use as the reference element.
+     * If this is provided, the menu will be inserted before this
+     * node in the host. The default is `null`, which will cause the
+     * menu to be added as the last child of the host.
+     */
+    ref?: HTMLElement;
+
+    /**
+     * The alignment of the menu.
+     *
+     * The default is `'left'` unless the document `dir` attribute is `'rtl'`
+     */
+    horizontalAlignment?: 'left' | 'right';
   }
 
   /**
@@ -1327,7 +1369,12 @@ export namespace Menu {
           if (!data.item.isEnabled) {
             aria['aria-disabled'] = 'true';
           }
-          aria.role = 'menuitem';
+          if (data.item.isToggled) {
+            aria.role = 'menuitemcheckbox';
+            aria['aria-checked'] = 'true';
+          } else {
+            aria.role = 'menuitem';
+          }
       }
       return aria;
     }
@@ -1535,7 +1582,10 @@ namespace Private {
     x: number,
     y: number,
     forceX: boolean,
-    forceY: boolean
+    forceY: boolean,
+    horizontalAlignment: 'left' | 'right',
+    host: HTMLElement | null,
+    ref: HTMLElement | null
   ): void {
     // Get the current position and size of the main viewport.
     const windowData = getWindowData();
@@ -1559,11 +1609,16 @@ namespace Private {
     style.maxHeight = `${maxHeight}px`;
 
     // Attach the menu to the document.
-    Widget.attach(menu, document.body);
+    Widget.attach(menu, host || document.body, ref);
 
     // Measure the size of the menu.
     let { width, height } = node.getBoundingClientRect();
 
+    // align the menu to the right of the target if requested or language is RTL
+    if (horizontalAlignment === 'right') {
+      x -= width;
+    }
+
     // Adjust the X position of the menu to fit on-screen.
     if (!forceX && x + width > px + cw) {
       x = px + cw - width;
