@@ -44,6 +44,7 @@ describe('@lumino/widgets', () => {
   beforeEach(() => {
     commands = new CommandRegistry();
     palette = new CommandPalette({ commands });
+    Widget.attach(palette, document.body);
   });
 
   afterEach(() => {
@@ -319,17 +320,308 @@ describe('@lumino/widgets', () => {
         MessageLoop.flush();
         expect(items()).to.have.length(0);
       });
+
+      it('should search a list of commands', () => {
+        // Add several commands to the command registry and the palette
+        commands.addCommand('example:cut', {
+          label: 'Cut',
+          mnemonic: 1,
+          iconClass: 'fa fa-cut',
+          execute: () => {
+            console.log('Cut');
+          }
+        });
+
+        commands.addCommand('example:copy', {
+          label: 'Copy File',
+          mnemonic: 0,
+          iconClass: 'fa fa-copy',
+          execute: () => {
+            console.log('Copy');
+          }
+        });
+
+        commands.addCommand('example:paste', {
+          label: 'Paste',
+          mnemonic: 0,
+          iconClass: 'fa fa-paste',
+          execute: () => {
+            console.log('Paste');
+          }
+        });
+
+        commands.addCommand('example:new-tab', {
+          label: 'New Tab',
+          mnemonic: 0,
+          caption: 'Open a new tab',
+          execute: () => {
+            console.log('New Tab');
+          }
+        });
+
+        commands.addCommand('example:close-tab', {
+          label: 'Close Tab',
+          mnemonic: 2,
+          caption: 'Close the current tab',
+          execute: () => {
+            console.log('Close Tab');
+          }
+        });
+
+        commands.addCommand('example:save-on-exit', {
+          label: 'Save on Exit',
+          mnemonic: 0,
+          caption: 'Toggle the save on exit flag',
+          execute: () => {
+            console.log('Save on Exit');
+          }
+        });
+
+        commands.addCommand('example:open-task-manager', {
+          label: 'Task Manager',
+          mnemonic: 5,
+          isEnabled: () => false,
+          execute: () => {}
+        });
+
+        commands.addCommand('example:close', {
+          label: 'Close',
+          mnemonic: 0,
+          iconClass: 'fa fa-close',
+          execute: () => {
+            console.log('Close');
+          }
+        });
+
+        commands.addCommand('example:one', {
+          label: 'One',
+          execute: () => {
+            console.log('One');
+          }
+        });
+
+        commands.addCommand('example:two', {
+          label: 'Two',
+          execute: () => {
+            console.log('Two');
+          }
+        });
+
+        commands.addCommand('example:three', {
+          label: 'Three',
+          execute: () => {
+            console.log('Three');
+          }
+        });
+
+        commands.addCommand('example:four', {
+          label: 'Four',
+          execute: () => {
+            console.log('Four');
+          }
+        });
+
+        commands.addCommand('example:black', {
+          label: 'Black',
+          execute: () => {
+            console.log('Black');
+          }
+        });
+
+        commands.addCommand('example:clear-cell', {
+          label: 'Clear Cell',
+          execute: () => {
+            console.log('Clear Cell');
+          }
+        });
+
+        commands.addCommand('example:cut-cells', {
+          label: 'Cut Cell(s)',
+          execute: () => {
+            console.log('Cut Cell(s)');
+          }
+        });
+
+        commands.addCommand('example:run-cell', {
+          label: 'Run Cell',
+          execute: () => {
+            console.log('Run Cell');
+          }
+        });
+
+        commands.addCommand('example:cell-test', {
+          label: 'Cell Test',
+          execute: () => {
+            console.log('Cell Test');
+          }
+        });
+
+        commands.addCommand('notebook:new', {
+          label: 'New Notebook',
+          execute: () => {
+            console.log('New Notebook');
+          }
+        });
+
+        commands.addKeyBinding({
+          keys: ['Accel X'],
+          selector: 'body',
+          command: 'example:cut'
+        });
+
+        commands.addKeyBinding({
+          keys: ['Accel C'],
+          selector: 'body',
+          command: 'example:copy'
+        });
+
+        commands.addKeyBinding({
+          keys: ['Accel V'],
+          selector: 'body',
+          command: 'example:paste'
+        });
+
+        commands.addKeyBinding({
+          keys: ['Accel J', 'Accel J'],
+          selector: 'body',
+          command: 'example:new-tab'
+        });
+
+        commands.addKeyBinding({
+          keys: ['Accel M'],
+          selector: 'body',
+          command: 'example:open-task-manager'
+        });
+
+        let palette = new CommandPalette({ commands });
+        Widget.attach(palette, document.body);
+        try {
+          palette.addItem({ command: 'example:cut', category: 'Edit' });
+          palette.addItem({ command: 'example:copy', category: 'Edit' });
+          palette.addItem({ command: 'example:paste', category: 'Edit' });
+          palette.addItem({ command: 'example:one', category: 'Number' });
+          palette.addItem({ command: 'example:two', category: 'Number' });
+          palette.addItem({ command: 'example:three', category: 'Number' });
+          palette.addItem({ command: 'example:four', category: 'Number' });
+          palette.addItem({ command: 'example:black', category: 'Number' });
+          palette.addItem({ command: 'example:new-tab', category: 'File' });
+          palette.addItem({ command: 'example:close-tab', category: 'File' });
+          palette.addItem({
+            command: 'example:save-on-exit',
+            category: 'File'
+          });
+          palette.addItem({
+            command: 'example:open-task-manager',
+            category: 'File'
+          });
+          palette.addItem({ command: 'example:close', category: 'File' });
+          palette.addItem({
+            command: 'example:clear-cell',
+            category: 'Notebook Cell Operations'
+          });
+          palette.addItem({
+            command: 'example:cut-cells',
+            category: 'Notebook Cell Operations'
+          });
+          palette.addItem({
+            command: 'example:run-cell',
+            category: 'Notebook Cell Operations'
+          });
+          palette.addItem({
+            command: 'example:cell-test',
+            category: 'Console'
+          });
+          palette.addItem({ command: 'notebook:new', category: 'Notebook' });
+          palette.id = 'palette';
+
+          // Search the command palette: update the inputNode, then force a refresh
+          palette.inputNode.value = 'clea';
+          palette.refresh();
+          MessageLoop.flush();
+
+          // Expect that headers and items appear in descending score order,
+          // even if the same header occurs multiple times.
+          const children = palette.contentNode.children;
+          expect(children.length).to.equal(7);
+          expect(children[0].textContent).to.equal('Notebook Cell Operations');
+          expect(children[1].getAttribute('data-command')).to.equal(
+            'example:clear-cell'
+          );
+          // The next match should be from a different category
+          expect(children[2].textContent).to.equal('File');
+          expect(children[3].getAttribute('data-command')).to.equal(
+            'example:close-tab'
+          );
+          // The next match should be the same as in a previous category
+          expect(children[4].textContent).to.equal('Notebook Cell Operations');
+          expect(children[5].getAttribute('data-command')).to.equal(
+            'example:cut-cells'
+          );
+          // The next match has the same category as the previous one did, so the header is not repeated
+          expect(children[6].getAttribute('data-command')).to.equal(
+            'example:run-cell'
+          );
+        } finally {
+          palette.dispose();
+        }
+      });
+
+      it('should clear the widget content if hidden', () => {
+        commands.addCommand('test', { execute: () => {}, label: 'test' });
+        palette.addItem({ command: 'test', category: 'test' });
+        const content = palette.contentNode;
+        const itemClass = '.lm-CommandPalette-item';
+        const items = () => content.querySelectorAll(itemClass);
+
+        palette.refresh();
+        MessageLoop.flush();
+
+        expect(items()).to.have.length(1);
+
+        palette.hide();
+        palette.refresh();
+        MessageLoop.flush();
+        expect(items()).to.have.length(0);
+      });
+
+      it('should clear the widget content if container is hidden', () => {
+        const parent = new Widget();
+        Widget.attach(parent, document.body);
+        try {
+          palette.parent = parent;
+          commands.addCommand('test', { execute: () => {}, label: 'test' });
+          palette.addItem({ command: 'test', category: 'test' });
+          const content = palette.contentNode;
+          const itemClass = '.lm-CommandPalette-item';
+          const items = () => content.querySelectorAll(itemClass);
+
+          palette.refresh();
+          MessageLoop.flush();
+
+          expect(items()).to.have.length(1);
+
+          parent.hide();
+          palette.refresh();
+          MessageLoop.flush();
+          expect(items()).to.have.length(0);
+        } finally {
+          parent.dispose();
+        }
+      });
     });
 
     describe('#handleEvent()', () => {
       it('should handle click, keydown, and input events', () => {
         let palette = new LogPalette({ commands });
         Widget.attach(palette, document.body);
-        ['click', 'keydown', 'input'].forEach(type => {
-          palette.node.dispatchEvent(new Event(type, { bubbles }));
-          expect(palette.events).to.contain(type);
-        });
-        palette.dispose();
+        try {
+          ['click', 'keydown', 'input'].forEach(type => {
+            palette.node.dispatchEvent(new Event(type, { bubbles }));
+            expect(palette.events).to.contain(type);
+          });
+        } finally {
+          palette.dispose();
+        }
       });
 
       context('click', () => {
@@ -338,7 +630,6 @@ describe('@lumino/widgets', () => {
           commands.addCommand('test', { execute: () => (called = true) });
 
           palette.addItem(defaultOptions);
-          Widget.attach(palette, document.body);
           MessageLoop.flush();
 
           let node = palette.contentNode.querySelector(
@@ -353,7 +644,6 @@ describe('@lumino/widgets', () => {
           commands.addCommand('test', { execute: () => (called = true) });
 
           palette.addItem(defaultOptions);
-          Widget.attach(palette, document.body);
           MessageLoop.flush();
 
           let node = palette.contentNode.querySelector(
@@ -370,7 +660,6 @@ describe('@lumino/widgets', () => {
           let content = palette.contentNode;
 
           palette.addItem(defaultOptions);
-          Widget.attach(palette, document.body);
           MessageLoop.flush();
 
           let node = content.querySelector('.lm-mod-active');
@@ -391,7 +680,6 @@ describe('@lumino/widgets', () => {
           let content = palette.contentNode;
 
           palette.addItem(defaultOptions);
-          Widget.attach(palette, document.body);
           MessageLoop.flush();
 
           let node = content.querySelector('.lm-mod-active');
@@ -413,7 +701,6 @@ describe('@lumino/widgets', () => {
           let content = palette.contentNode;
 
           palette.addItem(defaultOptions);
-          Widget.attach(palette, document.body);
           MessageLoop.flush();
 
           let node = content.querySelector('.lm-mod-active');
@@ -443,7 +730,6 @@ describe('@lumino/widgets', () => {
           let content = palette.contentNode;
 
           palette.addItem(defaultOptions);
-          Widget.attach(palette, document.body);
           MessageLoop.flush();
 
           expect(content.querySelector('.lm-mod-active')).to.equal(null);
@@ -470,7 +756,6 @@ describe('@lumino/widgets', () => {
             palette.addItem({ command: name, category: 'test' });
           });
 
-          Widget.attach(palette, document.body);
           MessageLoop.flush();
 
           let content = palette.contentNode;
@@ -500,7 +785,6 @@ describe('@lumino/widgets', () => {
             });
           });
 
-          Widget.attach(palette, document.body);
           MessageLoop.flush();
 
           let headers = () =>
