@@ -604,12 +604,75 @@ describe('@lumino/widgets', () => {
         );
       });
 
+      it('should accept horizontalAlignment menu flags', () => {
+        menu.addItem({ command: 'test' });
+        menu.open(300, 300, { horizontalAlignment: 'right' });
+        let { width } = menu.node.getBoundingClientRect();
+        const expectedX = Math.floor(300 - width);
+        expect(
+          menu.node.style.transform.startsWith(`translate(${expectedX}`)
+        ).to.equal(true);
+        expect(menu.node.style.transform.endsWith('px, 300px)')).to.equal(true);
+      });
+
+      it('horizontalAlignment should default to `right` if language direction is `rtl`', () => {
+        document.documentElement.setAttribute('dir', 'rtl');
+        menu.addItem({ command: 'test' });
+        menu.open(300, 300);
+        let { width } = menu.node.getBoundingClientRect();
+        const expectedX = Math.floor(300 - width);
+        expect(
+          menu.node.style.transform.startsWith(`translate(${expectedX}`)
+        ).to.equal(true);
+        expect(menu.node.style.transform.endsWith('px, 300px)')).to.equal(true);
+        document.documentElement.removeAttribute('dir'); // Reset the direction
+      });
+
       it('should bail if already attached', () => {
         menu.addItem({ command: 'test' });
         menu.open(10, 10);
         menu.open(100, 100);
         expect(menu.node.style.transform).to.equal('translate(10px, 10px)');
       });
+
+      it('should insert as last child under document.body by default', () => {
+        const div = document.body.appendChild(document.createElement('div'));
+        menu.addItem({ command: 'test' });
+        menu.open(10, 10);
+        expect(menu.node.parentElement).to.equal(document.body);
+        expect(menu.node.previousElementSibling).to.equal(div);
+        expect(menu.node.nextElementSibling).to.be.null;
+      });
+
+      it('should insert as last child under specified host element', () => {
+        const div = document.body.appendChild(document.createElement('div'));
+        const child = div.appendChild(document.createElement('div'));
+        menu.addItem({ command: 'test' });
+        menu.open(10, 10, { host: div });
+        expect(menu.node.parentElement).to.equal(div);
+        expect(menu.node.previousElementSibling).to.equal(child);
+        expect(menu.node.nextElementSibling).to.be.null;
+      });
+
+      it('should insert before reference element under document.body', () => {
+        const div1 = document.body.appendChild(document.createElement('div'));
+        const div2 = document.body.appendChild(document.createElement('div'));
+        menu.addItem({ command: 'test' });
+        menu.open(10, 10, { ref: div2 });
+        expect(menu.node.parentElement).to.equal(document.body);
+        expect(menu.node.previousElementSibling).to.equal(div1);
+        expect(menu.node.nextElementSibling).to.equal(div2);
+      });
+
+      it('should insert before reference element under specified host element', () => {
+        const div = document.body.appendChild(document.createElement('div'));
+        const child1 = div.appendChild(document.createElement('div'));
+        const child2 = div.appendChild(document.createElement('div'));
+        menu.open(10, 10, { host: div, ref: child2 });
+        expect(menu.node.parentElement).to.equal(div);
+        expect(menu.node.previousElementSibling).to.equal(child1);
+        expect(menu.node.nextElementSibling).to.equal(child2);
+      });
     });
 
     describe('#handleEvent()', () => {
@@ -774,8 +837,8 @@ describe('@lumino/widgets', () => {
           menu.node.dispatchEvent(
             new MouseEvent('mousemove', {
               bubbles,
-              clientX: rect.left,
-              clientY: rect.top
+              clientX: rect.left + 1,
+              clientY: rect.top + 1
             })
           );
           expect(menu.activeIndex).to.equal(0);
@@ -792,8 +855,8 @@ describe('@lumino/widgets', () => {
           menu.node.dispatchEvent(
             new MouseEvent('mousemove', {
               bubbles,
-              clientX: rect.left,
-              clientY: rect.top
+              clientX: rect.left + 1,
+              clientY: rect.top + 1
             })
           );
           expect(menu.activeIndex).to.equal(0);
@@ -843,8 +906,8 @@ describe('@lumino/widgets', () => {
           menu.node.dispatchEvent(
             new MouseEvent('mousemove', {
               bubbles,
-              clientX: rect.left,
-              clientY: rect.top
+              clientX: rect.left + 1,
+              clientY: rect.top + 1
             })
           );
           expect(menu.activeIndex).to.equal(0);
@@ -869,8 +932,8 @@ describe('@lumino/widgets', () => {
           menu.node.dispatchEvent(
             new MouseEvent('mousedown', {
               bubbles,
-              clientX: rect.left,
-              clientY: rect.top
+              clientX: rect.left + 1,
+              clientY: rect.top + 1
             })
           );
           expect(menu.isAttached).to.equal(true);
